/*******************************************/
/*  Raw Wave File Output Class,            */
/*  by Perry R. Cook, 1995-96              */ 
/*  This version opens a mono NeXT .snd    */
/*  file 16bit data at 22KHz, and          */
/*  pokes buffers of samples into it.      */
/*******************************************/

#include "RawWvOut.h"

#if defined(__NeXT_)

/********  NeXT Soundfile Header Struct   *******/
struct headerform {
    char pref[4];
    long hdr_length;
    long file_length;
    long mode;
    long samp_rate;
    long num_channels;
    char comment[1024];
};

RawWvOut :: RawWvOut(char *fileName)
{
    struct headerform hdr = {".sn",28,0,3,(long) SRATE,1,"TK"};
    char tempName[128];

    hdr.pref[3] = 'd';

    strcpy(tempName,fileName);
    strcat(tempName,".snd");

    fd = fopen(tempName,"wb");
    if (!fd)   {
	printf("Couldn't create soundfile %s  !!!!!!!!\n",fileName);
	exit(0);
    }
    fwrite(&hdr,4,7,fd);
    counter = 0;
    totalCount = 0;
}

RawWvOut :: ~RawWvOut()
{
    MY_FLOAT temp;
    fwrite(data,2,counter,fd);
    fseek(fd,8,SEEK_SET);
    fwrite(&totalCount,4,1,fd);
    fclose(fd);
    temp = (double) totalCount * ONE_OVER_SRATE;
    printf("%f Seconds Computed\n",temp);
}

long RawWvOut :: getCounter()
{
    return totalCount;
}

MY_FLOAT RawWvOut :: getTime()
{
    return (MY_FLOAT) totalCount * SRATE;
}

void RawWvOut :: tick(MY_FLOAT sample)
{

    data[counter++] = (short) (sample * 32000.0);
    totalCount += 1;
    if (counter == BUFFER_SIZE) {
	fwrite(data,2,BUFFER_SIZE,fd);
	counter = 0;
    }
}

#endif

#if defined(__SGI_REALTIME_)

/*******************************************/
/*  SGI Real-Time Wave File Output Class,  */
/*  by Perry R. Cook, 1996                 */ 
/*  This Object can opens the SGI soundout */
/*  device, and pokes buffers of samples   */
/*  into it.  The real code that does the  */
/*  is from Doug Scott of SGI.             */
/*******************************************/

#include "sgio.C"

RawWvOut :: RawWvOut(char *fileName)
{
    int fd;

    fd = init_sound(SRATE,1);
    if (fd<0)   {
	printf("Couldn't open SoundOut Device  !!!!!!!!\n");
	exit(0);
    }
    counter = 0;
}

RawWvOut :: ~RawWvOut()
{
    playbuf(data,counter);
    counter = 0;
    while (counter<BUFFER_SIZE) {
	data[counter++] = 0;
    }
    playbuf(data,counter);
    playbuf(data,counter);
    playbuf(data,counter);
    shuddup();
}

void RawWvOut :: tick(MY_FLOAT sample)
{

    data[counter++] = (short) (sample * 32000.0);
    if (counter == BUFFER_SIZE) {
	playbuf(data,counter);
	counter = 0;
    }
}

#endif

#if defined(__WIN95_REALTIME_)

/*******************************************/
/*  Windows95 Real-Time Wave Output Class, */
/*  by Perry R. Cook, 1996                 */ 
/*  This Object opens the WaveOut          */
/*  device, and pokes buffers of samples   */
/*  into it.  The real code that does the  */
/*  work is from Bill Putnam of CCRMA.     */
/*******************************************/

#include "waveout.c"

int which = 1;
short data1[BUFFER_SIZE];
short data2[BUFFER_SIZE];
short *data = data1;
short ready4More = 1;

short dataFunction( short* myData) {
	static BOOL state = FALSE;

	if (which==1)   {
		memcpy(myData, data1, BUFFER_SIZE*sizeof(short)*1 );
	    data = data2;
		which = 2;
	}
	else                    {
		memcpy(myData, data2, BUFFER_SIZE*sizeof(short)*1 );
	    data = data1;
		which = 1;
	}
	ready4More = 1;
	return 1;
}

RawWvOut :: RawWvOut(char *fileName)
{
    int fd;

	InitializeSpeaker(1,22050,BUFFER_SIZE,dataFunction);
    fd = 1;
	if (fd<0)   {
		printf("Couldn't open SoundOut Device  !!!!!!!!\n");
		exit(0);
    }
    counter = 0;
    StartPlaying();

}

RawWvOut :: ~RawWvOut()
{
	StopPlaying();
}

void RawWvOut :: tick(MY_FLOAT sample)
{

    while (!ready4More) {
//          sleep(100);
	}
	data[counter++] = (short) (sample * 32000.0);
	if (counter == BUFFER_SIZE) {
	    counter = 0;
		ready4More = 0;
    }
}

#endif

#if defined(__DOS_WAV_)

/********  Wav Soundfile Header Struct   *******/
struct wavehdr {char riff[4];
		long flength;
		char wave[4];
		char fmt[4];
		long format;
		short aone;
		short numchans;
		long srate;
		long bps;
		short numchans2;
		short bps2;
		char data[4];
		long dlength;
		};


RawWvOut :: RawWvOut(char *fileName)
{
    struct wavehdr hdr = {"RIF",36,"WAV","fmt",
			  16,1,1,22050,44100,1,16,"dat",0};

    hdr.riff[3] = 'F';
    hdr.wave[3] = 'E';
    hdr.fmt[3]  = ' ';
    hdr.data[3] = 'a';
    
    fd = fopen(fileName,"wb");
    if (!fd)   {
	printf("Couldn't create soundfile %s  !!!!!!!!\n",fileName);
	exit(0);
    }
    fwrite(&hdr,11,4,fd);
    counter = 0;
    totalCount = 0;
}

RawWvOut :: ~RawWvOut()
{
    MY_FLOAT temp;
    fwrite(data,2,counter,fd);

    temp = (double) totalCount * ONE_OVER_SRATE;
    totalCount *= 2;
    fseek(fd,40,SEEK_SET);
    fwrite(&totalCount,4,1,fd);
    totalCount += 8;
    fseek(fd,4,SEEK_SET);
    fwrite(&totalCount,4,1,fd);
    fclose(fd);
    printf("%f Seconds Computed\n",temp);
}

long RawWvOut :: getCounter()
{
    return totalCount;
}

MY_FLOAT RawWvOut :: getTime()
{
    return (MY_FLOAT) totalCount * SRATE;
}

void RawWvOut :: tick(MY_FLOAT sample)
{

    data[counter++] = (short) (sample * 32000.0);
    totalCount += 1;
    if (counter == BUFFER_SIZE) {
	fwrite(data,2,BUFFER_SIZE,fd);
	counter = 0;
    }
}

#endif


